@@ -2,6 +2,7 @@
 import random
 import time
 import urllib
+import re
 from hashlib import md5
 from functools import partial
 try:
@@ -25,22 +26,39 @@ COMPLEX_METHODS = ['secure', 'ads', 'messages', 'likes', 'friends',
 
 
 class VKError(Exception):
-    __slots__ = ["code", "description", "params"]
-    def __init__(self, code, description, params):
-        self.code, self.description, self.params = (code, description, params)
+    __slots__ = ["error"]
+    def __init__(self, error_data):
+        self.error = error_data
         Exception.__init__(self, str(self))
+
+    @property
+    def code(self):
+        return self.error['error_code']
+
+    @property
+    def description(self):
+        return self.error['error_msg']
+
+    @property
+    def params(self):
+        return self.error['request_params']
+
     def __str__(self):
         return "Error(code = '%s', description = '%s', params = '%s')" % (self.code, self.description, self.params)
 
-def _to_request_encoding(s):
+def _encode(s):
     if isinstance(s, unicode):
         return s.encode(REQUEST_ENCODING)
+
+    if isinstance(s, (dict, list, tuple)):
+        return json.dumps(s, ensure_ascii=False, encoding=REQUEST_ENCODING)
+
     return s # this can be number, etc.
 
 
 def signature(api_secret, params):
     keys = sorted(params.keys())
-    param_str = "".join(["%s=%s" % (str(key), _to_request_encoding(params[key])) for key in keys])
+    param_str = "".join(["%s=%s" % (str(key), _encode(params[key])) for key in keys])
     return md5(param_str + str(api_secret)).hexdigest()
 
 # We have to support this:
@@ -66,11 +84,15 @@ class _API(object):
     def _get(self, method, timeout=DEFAULT_TIMEOUT, **kwargs):
         status, response = self._request(method, timeout=timeout, **kwargs)
         if not (200 <= status <= 299):
-            raise VKError(status, "HTTP error", kwargs)
+            raise VKError({
+                'error_code': status,
+                'error_msg': "HTTP error",
+                'request_params': kwargs,
+            })
 
         data = json.loads(response)
         if "error" in data:
-            raise VKError(data["error"]["error_code"], data["error"]["error_msg"], data["error"]["request_params"])
+            raise VKError(data["error"])
         return data['response']
 
     def __getattr__(self, name):
@@ -97,7 +119,7 @@ class _API(object):
     def _request(self, method, timeout=DEFAULT_TIMEOUT, **kwargs):
 
         for key, value in kwargs.iteritems():
-            kwargs[key] = _to_request_encoding(value)
+            kwargs[key] = _encode(value)
 
         if self.token:
             # http://vkontakte.ru/developers.php?oid=-1&p=Выполнение_запросов_к_API
@@ -123,7 +145,9 @@ class _API(object):
             url = API_URL
             secure = False
         data = urllib.urlencode(params)
-        headers = {"Accept": "application/json", "Content-Type": "application/x-www-form-urlencoded"}
+
+        headers = {"Accept": "application/json",
+                   "Content-Type": "application/x-www-form-urlencoded"}
 
         # urllib2 doesn't support timeouts for python 2.5 so
         # custom function is used for making http requests
